(define-module (ui deploy ajax)
    :use-module (alterator ajax)
    :use-module (alterator woo)
    :use-module (alterator algo)
    :use-module (ice-9 regex)
    :export (init))

;;; Functions

(define (get-parameter-name parameter)
  (let ((m (string-match "^([^=]+)=.*" parameter)))
    (if m
        (regexp-substitute #f (string-match "^([^=]+)=.*" parameter) 'pre 1)
        "")))

(define (get-parameter-value parameter)
  (let ((m (string-match "^([^=]+)=.*" parameter)))
    (if m
      (regexp-substitute #f (string-match "^[^=]+=(.*)" parameter) 'pre 1)
      "")))

(define (update-parameter parameter-pair)
  (let ((parameter (get-parameter-name parameter-pair))
        (value (get-parameter-value parameter-pair)))
    (form-update-value "value" value)
    (form-update-value "parameter_label" (string-append parameter ":"))))

(define (update-parameters-only role parameter-pair)
  (let* ((parameters-data (woo-list "/deploy/parameters" 'role role)))
    (format #t "Parameters:~a\n" parameters-data)
    (form-update-enum "parameters" parameters-data)
    (form-update-value "parameters" parameter-pair)
    (format #t "Param: ~a\n" (form-value "parameters"))))

(define (update-parameters role)
  (let* ((parameters-data (woo-list "/deploy/parameters" 'role role))
         (parameter-pair (woo-get-option (cond-car parameters-data) 'name)))
    (form-update-enum "parameters" parameters-data)
    (form-update-value "parameters" parameter-pair)
    (update-parameter parameter-pair)))

(define (ui-update-for-role)
  (catch/message
  	(lambda()
      (let* ((role (form-value "roles")))
        (form-update-value "role" role)
        (update-parameters role)))))

(define (ui-update-for-parameter)
  (catch/message
  	(lambda()
      (let* ((parameter-pair (form-value "parameters")))
        (update-parameter parameter-pair)))))

(define (ui-list)
  (catch/message
    (lambda()
      (let* ((roles-data (woo-list "/deploy"))
             (role (woo-get-option (cond-car roles-data) 'name))
             (status (woo-get-option (woo-read-first "/deploy/status")
                                     'status)))
        (form-update-enum "roles" roles-data)
        (form-update-value "role" role)
        (form-update-value "status" status)
        (update-parameters role)))))

;;; Buttons

(define (reset-all)
  (let* ((role (form-value "roles")))
    (catch/message
      (lambda()
        (woo-write "/deploy/reset_all" 'role role)
        (update-parameters role)))))

(define (set)
  (let* ((role (form-value "roles"))
         (parameter-pair (form-value "parameters"))
         (parameter (get-parameter-name parameter-pair))
         (value (form-value "value")))
    (catch/message
      (lambda()
        (format #t "Role: ~a\n" role)
        (format #t "Parameter: ~a\n" parameter)
        (format #t "Value: ~a\n" value)
        (woo-write "/deploy/set"
                   'role role
                   'parameter parameter
                   'value value)
        (update-parameters-only role (string-append parameter "=" value))))))

(define (reset)
  (let* ((role (form-value "roles"))
         (parameter-pair (form-value "parameters"))
         (parameter (get-parameter-name parameter-pair))
         (value (woo-get-option (woo-read-first "/deploy"
                                                'role role
                                                'parameter parameter)
                                'value)))
    (catch/message
      (lambda()
        (woo-write "/deploy/reset" 'role role 'parameter parameter)
        (update-parameters-only role (string-append parameter "=" value))
        (form-update-value "value" value)))))

(define (cancel)
  (let* ((role (form-value "roles"))
         (parameter-pair (form-value "parameters"))
         (parameter (get-parameter-name parameter-pair))
         (value (get-parameter-value parameter-pair)))
    (catch/message
      (lambda()
        (form-update-value "value" value)))))

(define (deploy)
  (let ((role (form-value "roles")))
    (catch/message
      (lambda()
        (woo-write "/deploy" 'role role)
        (update-parameters role)
        (form-update-value "status"
                           (woo-get-option (woo-read-first "/deploy/status")
                                           'status))))))

;;;

(define (init)
  (ui-list)
  (form-bind "roles" "change" ui-update-for-role)
  (form-bind "parameters" "change" ui-update-for-parameter)
  (form-bind "reset_all" "click" reset-all)
  (form-bind "set" "click" set)
  (form-bind "reset" "click" reset)
  (form-bind "cancel" "click" cancel)
  (form-bind "deploy" "click" deploy))
