(document:surround "/std/frame")

(vbox
  (hbox align "left"
    (label text "Role:") (combobox name "roles" enumref "/deploy")
    (label text "Status:") (label "" name "status"))

  ; (separator)

  (gridbox
    (vbox
      (hbox
        (label text "Parameters")
        (button (_ "Reset all") name "reset_all"))
      (listbox columns 2
               ; row '#((parameter . "") (value . ""))
               ; header (vector (_ "Parameter") (_ "Value"))
               name "parameters")
      (hbox
        (label "" name "parameter_label")
        (edit name "value_edit")
        (button (_ "Set") name "set")
        (button (_ "Reset") name "reset")
        (button (_ "Cancel") name "cancel"))))

  ;; Without this form value does not work
  (label "" name "value" visibility #f)
  (label "" name "role" visibility #f)
  (label "" name "parameter" visibility #f)

  (button (_ "Deploy") name "deploy"))

;;; Functions

(define (get-parameter-name parameter)
  (let ((m (string-match "^([^=]+)=.*" parameter)))
    (if m
        (regexp-substitute #f (string-match "^([^=]+)=.*" parameter) 'pre 1)
        "")))

(define (get-parameter-value parameter)
  (let ((m (string-match "^([^=]+)=.*" parameter)))
    (if m
      (regexp-substitute #f (string-match "^[^=]+=(.*)" parameter) 'pre 1)
      "")))

(define (update-parameter parameter-pair)
  (let* ((parameter (get-parameter-name parameter-pair))
         (value (get-parameter-value parameter-pair)))
    (form-update-value "value" value)
    (form-update-value "value_edit" value)
    (form-update-value "parameter_label" (string-append parameter ":"))
    (form-update-value "parameter" parameter)))

(define (update-parameters-only role)
  (let* ((parameters-data (woo-list "/deploy/parameters" 'role role)))
    (form-update-enum "parameters" parameters-data)))

(define (update-parameters role)
  (let* ((parameters-data (woo-list "/deploy/parameters" 'role role))
         (parameter-pair (woo-get-option (cond-car parameters-data)
                                         'name)))
    (form-update-enum "parameters" parameters-data)
    (update-parameter parameter-pair)))

(define (ui-update-for-role)
  (catch/message
  	(lambda()
      (let* ((role (form-value "roles")))
        (form-update-value "role" role)
        (update-parameters role)))))

(define (ui-update-for-parameter)
  (catch/message
  	(lambda()
      (let* ((parameter-pair (form-value "parameters")))
        (update-parameter parameter-pair)))))

(define (ui-list)
  (catch/message
    (lambda()
      (let* ((roles-data (woo-list "/deploy"))
             (role (woo-get-option (cond-car roles-data) 'name))
             (status (woo-get-option (woo-read-first "/deploy/status")
                                     'status)))
        (form-update-enum "roles" roles-data)
        (form-update-value "role" role)
        (form-update-value "status" status)
        (update-parameters role)))))

;;; Buttons

(define (reset-all)
  (let* ((role (form-value "role")))
    (catch/message
      (lambda()
        (woo-write "/deploy/reset_all" 'role role)
        (update-parameters role)))))

(define (set)
  (let ((role (form-value "role"))
        (parameter (form-value "parameter"))
        (value (form-value "value_edit")))
    (catch/message
      (lambda()
        (woo-write "/deploy/set"
                   'role role
                   'parameter parameter
                   'value value)
        (update-parameters-only role)
        (form-update-value "value" value)))))

(define (reset)
  (let* ((role (form-value "role"))
         (parameter (form-value "parameter"))
         (value (woo-get-option (woo-read-first "/deploy"
                                                'role role
                                                'parameter parameter)
                                'value)))
    (catch/message
      (lambda()
        (woo-write "/deploy/reset" 'role role 'parameter parameter)
        (update-parameters-only role)
        (form-update-value "value_edit" value)
        (form-update-value "value" value)))))

(define (cancel)
  (let ((role (form-value "role"))
        (parameter (form-value "parameter"))
        (value (form-value "value")))
    (catch/message
      (lambda()
        (form-update-value "value_edit" value)))))

(define (deploy)
  (let ((role (form-value "role")))
    (catch/message
      (lambda()
        (woo-write "/deploy" 'role role)
        (update-parameters role)
        (form-update-value "status"
                           (woo-get-option (woo-read-first "/deploy/status")
                                           'status))))))

;;;

(define (ui-init)
  (ui-list)
  (form-bind "roles" "change" ui-update-for-role)
  (form-bind "parameters" "change" ui-update-for-parameter)
  (form-bind "reset_all" "click" reset-all)
  (form-bind "set" "click" set)
  (form-bind "reset" "click" reset)
  (form-bind "cancel" "click" cancel)
  (form-bind "deploy" "click" deploy))

(document:root (when loaded (ui-init)))
